VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DX3D"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Private Declare Function timeGetTime Lib "winmm.dll" () As Long
Dim nStartTime As Long
Dim nEndTime As Long
Dim FPS, frames As Integer, frame_speed As Single

Public Dx As DirectX7                    'Direct X
Public DD As DirectDraw7                 'Direct Draw
Public Primary As DirectDrawSurface7     'Primary surface
Public BackBuffer As DirectDrawSurface7  'Backbuffer
Public Direct3D As Direct3D7             'Direct 3D
Public Device As Direct3DDevice7         'Direct 3D Device
Private Type LightSource
Red As Single
Green As Single
Blue As Single
AlhaVal As Single
End Type
Dim MapX As Integer
Dim MapY As Integer

Public Sub InitializeDX(MForm As Form, Optional Width As Integer = 640, Optional Height As Integer = 480, Optional BPP As Byte = 16)
    Dim ddsd As DDSURFACEDESC2
    Dim caps As DDSCAPS2
    Dim DEnum As Direct3DEnumDevices
    Dim Guid As String
    
    ' Create the DirectDraw object and set the application
    ' cooperative level.
    Set Dx = New DirectX7
    Set DD = Dx.DirectDrawCreate("")
    
    DD.SetCooperativeLevel MForm.hWnd, DDSCL_EXCLUSIVE Or DDSCL_FULLSCREEN Or DDSCL_ALLOWREBOOT
    DD.SetDisplayMode Width, Height, BPP, 0, DDSDM_DEFAULT
    
    ' Prepare and create the primary surface.
    ddsd.lFlags = DDSD_BACKBUFFERCOUNT Or DDSD_CAPS
    ddsd.ddsCaps.lCaps = DDSCAPS_COMPLEX Or DDSCAPS_FLIP Or DDSCAPS_3DDEVICE Or DDSCAPS_PRIMARYSURFACE
    ddsd.lBackBufferCount = 1
    
    Set Primary = DD.CreateSurface(ddsd)
    
    'Attach the backbuffer. the DDSCAPS_3DDEVICE tells the
    'backbuffer that its to be used for 3D stuff
    caps.lCaps = DDSCAPS_BACKBUFFER Or DDSCAPS_3DDEVICE
    Set BackBuffer = Primary.GetAttachedSurface(caps)

    
    Set Direct3D = DD.GetDirect3D
    
    'Get the last device driver (last one is usually the best one)
    Set DEnum = Direct3D.GetDevicesEnum
    Guid = DEnum.GetGuid(DEnum.GetCount)
    
    'Set the Device
    Set Device = Direct3D.CreateDevice(Guid, BackBuffer)
    
    'Our blending states.
    'you can change D3DBLEND_ONE, check the DX7 SDK for other values
    Device.SetRenderState D3DRENDERSTATE_DESTBLEND, D3DBLEND_INVSRCALPHA
    Device.SetRenderState D3DRENDERSTATE_SRCBLEND, D3DBLEND_SRCALPHA
    Device.SetTextureStageState 0, D3DTSS_MAGFILTER, 2
    Device.SetTextureStageState 0, D3DTSS_MINFILTER, 2
    End Sub

Public Function EndDX(MForm As Form)
    Call DD.RestoreDisplayMode
    Call DD.SetCooperativeLevel(MForm.hWnd, DDSCL_NORMAL)
End Function

Public Function CreateTextureSurface(sFile As String, w As Long, h As Long, Optional ColKey As Integer = 0) As DirectDrawSurface7

    Dim bOK As Boolean
    Dim enumTex As Direct3DEnumPixelFormats
    Dim sLoadFile As String
    Dim I As Long
    Dim ddsd As DDSURFACEDESC2
    Dim SurfaceObject As DirectDrawSurface7
    Dim Init As Boolean

    ddsd.lFlags = DDSD_CAPS Or DDSD_TEXTURESTAGE Or DDSD_PIXELFORMAT
    If ((h <> 0) And (w <> 0)) Then
        ddsd.lFlags = ddsd.lFlags Or DDSD_HEIGHT Or DDSD_WIDTH
        ddsd.lHeight = h
        ddsd.lWidth = w
    End If
    
    Set enumTex = Device.GetTextureFormatsEnum()

    'check if device supports 16bit surfaces
    For I = 1 To enumTex.GetCount()
        bOK = True
        Call enumTex.GetItem(I, ddsd.ddpfPixelFormat)

        With ddsd.ddpfPixelFormat
            If .lRGBBitCount <> 16 Then bOK = False
        End With
        If bOK = True Then Exit For
    Next

    If bOK = False Then
        Debug.Print "Unable to find 16bit surface support on your hardware - exiting"
        Init = False
    End If

    'set some texture surface flags
    If Device.GetDeviceGuid() = "IID_IDirect3DHALDevice" Then
            ddsd.ddsCaps.lCaps = DDSCAPS_TEXTURE
            ddsd.ddsCaps.lCaps2 = DDSCAPS2_TEXTUREMANAGE
            ddsd.lTextureStage = 0
    Else
            ddsd.ddsCaps.lCaps = DDSCAPS_TEXTURE
            ddsd.ddsCaps.lCaps2 = 0
            ddsd.lTextureStage = 0
    End If

    'If no filename was passed then create a blank surface
    If sFile = "" Then
        Set SurfaceObject = DD.CreateSurface(ddsd)
    Else
        Set SurfaceObject = DD.CreateSurfaceFromFile(sFile, ddsd)
    End If

    Set CreateTextureSurface = SurfaceObject

    'Colour key
    Dim ddckColourKey As DDCOLORKEY
    Dim ddpf As DDPIXELFORMAT
    If ColKey > 0 Then 'Black colorkey
        ddckColourKey.low = 0
        ddckColourKey.high = 0
        CreateTextureSurface.SetColorKey DDCKEY_SRCBLT, ddckColourKey
    End If
End Function

Public Sub Blt3D(tmpSurface As DirectDrawSurface7, Vertex() As D3DTLVERTEX)
    Call Device.SetTexture(0, tmpSurface)
    Call Device.DrawPrimitive(D3DPT_TRIANGLESTRIP, D3DFVF_TLVERTEX, Vertex(0), 4, D3DDP_DEFAULT)
End Sub

Public Sub Clear_Device()
    Dim ClearRect(0 To 0) As D3DRECT
    ClearRect(0).X2 = 640
    ClearRect(0).Y2 = 480
    Device.Clear 1, ClearRect, D3DCLEAR_TARGET, 0, 0, 0
End Sub

Public Function BeginScene()
Device.BeginScene
End Function

Public Function EndScene()
Device.EndScene
End Function

Public Sub EnableBlendOne()
    
    Device.SetRenderState D3DRENDERSTATE_ALPHABLENDENABLE, False
    Device.SetRenderState D3DRENDERSTATE_ALPHABLENDENABLE, True
    
    Device.SetRenderState D3DRENDERSTATE_SRCBLEND, D3DBLEND_ONE
    Device.SetRenderState D3DRENDERSTATE_DESTBLEND, D3DBLEND_ONE
        
End Sub

Public Sub DisableBlendOne()
    
    Device.SetRenderState D3DRENDERSTATE_ALPHABLENDENABLE, False
            
End Sub

Public Sub Set_ColorKey(BlendOn As Boolean)
    Device.SetRenderState D3DRENDERSTATE_COLORKEYENABLE, BlendOn
End Sub

Public Sub DDBltFast(Surface As DirectDrawSurface7, RECTvar As RECT, X As Integer, Y As Integer, Optional transparent As Boolean = True, Optional Clip As Boolean = True)
    'This subroutine will BltFast a surface to the
    'backbuffer.
    
    'CLIPPING
    'Temporary rect
    Dim RectTEMP As RECT
    RectTEMP = RECTvar
    
    If Clip = True Then
        'Set up screen rect for clipping
        Dim ScreenRect As RECT
        With ScreenRect
            .Top = Y
            .Left = X
            .Bottom = Y + RECTvar.Bottom
            .Right = X + RECTvar.Right
        End With
        'Clip surface
        With ScreenRect
            If .Bottom > 480 Then
                RectTEMP.Bottom = RectTEMP.Bottom - (.Bottom - 480)
                .Bottom = 590
            End If
            If .Left < 0 Then
                RectTEMP.Left = RectTEMP.Left - .Left
                .Left = 0
                X = 0
            End If
            If .Right > 640 Then
                RectTEMP.Right = RectTEMP.Right - (.Right - 640)
                .Right = 640
            End If
            If .Top < 0 Then
                RectTEMP.Top = RectTEMP.Top - .Top
                .Top = 0
                Y = 0
            End If
        End With
    
    End If
    
    If transparent = False Then
        Call BackBuffer.BltFast(X, Y, Surface, RectTEMP, DDBLTFAST_WAIT)
    Else
        Call BackBuffer.BltFast(X, Y, Surface, RectTEMP, DDBLTFAST_WAIT Or DDBLTFAST_SRCCOLORKEY)
    End If
End Sub

Public Sub DDCreateSurface(Surface As DirectDrawSurface7, BmpPath As String, RECTvar As RECT, Optional TransCol As Integer = 0, Optional UseSystemMemory As Boolean = True)
    'This sub will load a bitmap from a file
    'into a specified dd surface. Transparent
    'colour is black (0) by default.
    
    Dim tempddsd As DDSURFACEDESC2
    
    Set Surface = Nothing
    
    'Load sprite
    tempddsd.lFlags = DDSD_CAPS
    If UseSystemMemory = True Then
        tempddsd.ddsCaps.lCaps = DDSCAPS_SYSTEMMEMORY Or DDSCAPS_OFFSCREENPLAIN
    End If
    Set Surface = DD.CreateSurfaceFromFile(BmpPath, tempddsd)
    
    'set the RECT dimensions
    RECTvar.Right = tempddsd.lWidth
    RECTvar.Bottom = tempddsd.lHeight
    'Colour key
    Dim ddckColourKey As DDCOLORKEY
    ddckColourKey.low = TransCol
    ddckColourKey.high = TransCol
    Surface.SetColorKey DDCKEY_SRCBLT, ddckColourKey
    
    
End Sub

'    Call DX.CreateD3DTLVertex(X1, Y1, 0, 1, color1, 1, 0, 0, Vertex(0))
 '   Call DX.CreateD3DTLVertex(X2, Y2, 0, 1, color2, 1, 1, 0, Vertex(1))
  '  Call DX.CreateD3DTLVertex(X4, Y4, 0, 1, color4, 1, 0, 1, Vertex(2))
   ' Call DX.CreateD3DTLVertex(X3, Y3, 0, 1, color3, 1, 1, 1, Vertex(3))


Public Function SetVertex(Vertex() As D3DTLVERTEX, X As Integer, Y As Integer, Width As Integer, Height As Integer, Optional color1 As Long = &HFFFFFF, Optional color2 As Long = &HFFFFFF, Optional color3 As Long = &HFFFFFF, Optional color4 As Long = &HFFFFFF)
    Call Dx.CreateD3DTLVertex(X, Y, 0, 1, color1, 1, 0, 0, Vertex(0))
    Call Dx.CreateD3DTLVertex(X + Width, Y, 0, 1, color2, 1, 1, 0, Vertex(1))
    Call Dx.CreateD3DTLVertex(X, Y + Height, 0, 1, color4, 1, 0, 1, Vertex(2))
    Call Dx.CreateD3DTLVertex(X + Width, Y + Height, 0, 1, color3, 1, 1, 1, Vertex(3))
End Function
Public Function ISoRotateVertex(Vertex() As D3DTLVERTEX, X As Integer, Y As Integer, Width As Integer, Height As Integer, Angle As Integer, Optional color1 As Long = &HFFFFFF, Optional color2 As Long = &HFFFFFF, Optional color3 As Long = &HFFFFFF, Optional color4 As Long = &HFFFFFF)
If Angle <= 1 Then
    Call Dx.CreateD3DTLVertex(X, Y, 0, 1, color1, 1, 0, 0, Vertex(0))
    Call Dx.CreateD3DTLVertex(X + Width + Width / 2, Y + Height / 2, 0, 1, color2, 1, 1, 0, Vertex(1))
    Call Dx.CreateD3DTLVertex(X - Width, Y + Height, 0, 1, color4, 1, 0, 1, Vertex(2))
    Call Dx.CreateD3DTLVertex(X + Width - Width / 2, Y + Height + Height / 2, 0, 1, color3, 1, 1, 1, Vertex(3))
ElseIf Angle = 2 Then
    Call Dx.CreateD3DTLVertex(X, Y - Height / Height, 0, 1, color1, 1, 0, 0, Vertex(0))
    Call Dx.CreateD3DTLVertex(X + Width + Width / 2, Y + (Height / 2), 0, 1, color2, 1, 1, 0, Vertex(1))
    Call Dx.CreateD3DTLVertex(X, Y + Height / 2 * 2 + Height / 2, 0, 1, color4, 1, 0, 1, Vertex(2))
    Call Dx.CreateD3DTLVertex(X + Width + Width / 2, Y + Height / 2 * 2 + Height, 0, 1, color3, 1, 1, 1, Vertex(3))
ElseIf Angle = 3 Then
    Call Dx.CreateD3DTLVertex(X - Width / 2, Y, 0, 1, color1, 1, 0, 0, Vertex(0))
    Call Dx.CreateD3DTLVertex(X + Width - Width / 2, Y - (Height), 0, 1, color2, 1, 1, 0, Vertex(1))
    Call Dx.CreateD3DTLVertex(X - Width / 2, Y + Height + Height / 2, 0, 1, color4, 1, 0, 1, Vertex(2))
    Call Dx.CreateD3DTLVertex(X + Width - Width / 2, Y + Height - Height / 2, 0, 1, color3, 1, 1, 1, Vertex(3))
End If
End Function

Public Function RotateVertex(Vertex() As D3DTLVERTEX, X As Integer, Y As Integer, Width As Integer, Height As Integer, Angle As Double, Optional color1 As Long = &HFFFFFF, Optional color2 As Long = &HFFFFFF, Optional color3 As Long = &HFFFFFF, Optional color4 As Long = &HFFFFFF)
Angle = Angle + 10
    Dim v As D3DVECTOR
    Dim TempVector As D3DVECTOR
    TempVector.X = -Width / 2
    TempVector.Y = -Height / 2
    v.X = TempVector.X * Cos(-Angle * 3.14159 / 180) + TempVector.Y * -Sin(-Angle * 3.14159 / 180)
    v.Y = TempVector.X * Sin(-Angle * 3.14159 / 180) + TempVector.Y * Cos(-Angle * 3.14159 / 180)
Dx.CreateD3DTLVertex v.X + Width / 2 + X, -v.Y + Height / 2 + Y, 0, 1, color1, 1, 0, 1, Vertex(0)
    TempVector.X = -Width / 2
    TempVector.Y = Height / 2
    v.X = TempVector.X * Cos(-Angle * 3.14159 / 180) + TempVector.Y * -Sin(-Angle * 3.14159 / 180)
    v.Y = TempVector.X * Sin(-Angle * 3.14159 / 180) + TempVector.Y * Cos(-Angle * 3.14159 / 180)
        Dx.CreateD3DTLVertex v.X + Width / 2 + X, -v.Y + Height / 2 + Y, 0, 1, color2, 1, 0, 0, Vertex(1)
    TempVector.X = Width / 2
    TempVector.Y = -Height / 2
    v.X = TempVector.X * Cos(-Angle * 3.14159 / 180) + TempVector.Y * -Sin(-Angle * 3.14159 / 180)
    v.Y = TempVector.X * Sin(-Angle * 3.14159 / 180) + TempVector.Y * Cos(-Angle * 3.14159 / 180)
    Dx.CreateD3DTLVertex v.X + Width / 2 + X, -v.Y + Height / 2 + Y, 0, 1, color4, 1, 1, 1, Vertex(2)
    TempVector.X = Width / 2
    TempVector.Y = Height / 2
    v.X = TempVector.X * Cos(-Angle * 3.14159 / 180) + TempVector.Y * -Sin(-Angle * 3.14159 / 180)
    v.Y = TempVector.X * Sin(-Angle * 3.14159 / 180) + TempVector.Y * Cos(-Angle * 3.14159 / 180)
    Dx.CreateD3DTLVertex v.X + Width / 2 + X, -v.Y + Height / 2 + Y, 0, 1, color3, 1, 1, 0, Vertex(3)
End Function

Public Function MakeRect(Left As Integer, Top As Integer, Right As Integer, Bottom As Integer) As RECT
With MakeRect
.Left = Left
.Right = Right
.Top = Top
.Bottom = Bottom

End With
End Function

Public Function DDFlip()
Primary.Flip Nothing, DDFLIP_WAIT
End Function

Public Function DrawFPS()

        nStartTime = timeGetTime()
        frames = frames + 1
        If FPS = 0 Then FPS = 500
        If (nStartTime - nEndTime) > 1000 Then
            FPS = frames
                        On Local Error Resume Next
            frame_speed = 0.027 / FPS '0.027 is the speed of update
            frames = 0

                        nEndTime = timeGetTime()
            End If
BackBuffer.DrawText 5, 25, "FPS " & FPS, False
End Function

Public Function ClearScreen()
Dim Rct As RECT
With Rct
.Bottom = 480
.Right = 640
End With
BackBuffer.BltColorFill Rct, 0
End Function
